#!/usr/bin/perl
#
# Echo cleaner filter for HPT V1.0 beta.
#
# (c) 2019 Fabio Bizzi - 2:335/364 - http://mimac.bizzi.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# This filter was born to help sysops or node owners to keep clean
# the Echo Areas from unwanted messages, it offers filter options
# and search options.
#
# You can choose to kill selected messages or to move them in a 
# proper Echo area (BAD in the configuration example).
#
# The filter reports positive matches in the hpt logfile and looks
# for his configuration file in the directory referenced by the
# FIDOCONFIG enviromnent variable, IF THE FIDOCONFIG ENVIRONMENT
# VARIABLE IS NOT SET YOU HAVE TO CHANGE THE STATICCONFIG CONSTANT
# TO POINT TO YOUR echocleaner.cfg FILE.
#
# You can take a look at the example configuration file for a list of
# options and search keys.
#
# This is an example configuration to run the filter
#
# Insert into config:
# hptperlfile /home/fido/lib/filter.pl
# and place to filter.pl some like this:
# BEGIN { require "echocleaner.pl"; }
# sub filter{
#   &echocleaner;
# }
#

use warnings;
use v5.10;

# UPDATE THIS!!!!
use constant STATICCONFIG => "/home/husky/etc/fido/echocleaner.cfg";

no warnings 'experimental::smartmatch';

sub w_log;

my $riga;
my @spl;
my @spl2;
my $i;
my $field_from_to = "NULL";
my $field_node = "NULL";
my $field_area = "NULL";
my $field_nuke = "KILL";
my $field_oblivion = "NO";
my $flag_area;
my $flag_from_to;
my $flag_addr;
my $origin;
my $start;
my $stop;
my $temp;
my $config;

sub echocleaner()
{
 # Uncomment for debugging
 # w_log('C',"echocleaner.pl Start");
 # w_log('C',$area);
 # w_log('C',$fromname);
 # w_log('C',$fromaddr);

 # Check if the message is a Netmail or Echomail

 if (defined($area)){

   # Get the configuration file, lookig to FIDOCONFIG directory or using
   # the hardcoded path.

   if ($ENV{'FIDOCONFIG'}){
     $stop = index $ENV{'FIDOCONFIG'}, "config" ,0;
     $temp = substr $ENV{'FIDOCONFIG'}, 0, ($stop);
     $config = $temp . "echocleaner.cfg";
   }else{
      $config = STATICCONFIG;
   }

   # Open the configuration file

   if (!(open (FILE, '<', $config))){
     w_log('1',"Error configuration file echocleaner.cfg not found!!!!");
     return"";
     die;
   }
 
   # Get the originating node from the *Origin line

   $start = index $text, "Origin:";
   $stop = index $text, ")" ,$start;
   $temp = substr $text, $start, ($stop-$start+1);
   $start = index($temp,"(");
   $stop = index($temp,")");
   $start = $start +1;
   $origin = substr $temp, $start, ($stop-$start);
   # Uncomment for debugging
   # w_log('C',"Origin: $origin");

   # Enter in the file loop to get the config

   while (<FILE>) {
     chomp ($riga = $_);

     # If the row is null jump to the next row

     if ($riga ne ""){
     
       # If the row is # so is a remark and jump to the next row

       if ((substr $riga,0,1) ne "#"){
         @spl = split(',',$riga);

         # Get the single parameter

         foreach $i (@spl)
         {
           @spl2 = split('=',$i);
           given(uc($spl2[0])){
             when ("AREA") { $field_area = uc($spl2[1]); }
             when ("NAME") { $field_from_to = uc($spl2[1]); }
             when ("NODE") { $field_node = $spl2[1]; }
             when ("ACTION") {$field_nuke = uc($spl2[1]); }
             when ("OBLIVION") {$field_oblivion = uc($spl2[1]); }
           }
         }

         # Set the flags
         #
         # Check the Area

         if ($field_area eq "NULL"){
         $flag_area="OK";
         }
         elsif ($area eq $field_area){
             $flag_area="OK";
         }
         else{
           $flag_area="KO";
         }

         # Check The Name - From+To or only From

         if ($field_from_to eq "NULL"){
           $flag_from_to="OK";
         }
         elsif (($field_oblivion eq "YES") && (((uc($fromname)) eq $field_from_to) || (((uc($toname)) eq $field_from_to)))) {
             $flag_from_to="OK";
         }
         elsif (($field_oblivion ne "YES") && ((uc($fromname)) eq $field_from_to)){
             $flag_from_to="OK";
         }
         else{
           $flag_from_to="KO";
         }

         # Check The Address, if OBLIVION is set the address is not useful.

         if (($field_node eq "NULL") || ($field_oblivion eq "YES")){
           $flag_addr="OK";
         }
         elsif ($origin eq $field_node){
             $flag_addr="OK";
         }
         else{
           $flag_addr="KO";
         }

         # Final Check if the message is unwanted

         if (($flag_area eq "OK") && ($flag_from_to eq "OK") && ($flag_addr eq "OK")){
           # Check if we want to delete the message or to move in another echo

           if ($field_nuke ne "KILL"){
             putMsgInArea($field_nuke, $fromname, $toname, $fromaddr, "",
                          $subject, $date, $attr, $text, 0);
           }else{
              $kill = 1;
           }
           return "EchoCleaner Positive";
         }

         # Reset the running variables

         $field_area = "NULL";
         $field_from_to = "NULL";
         $field_node = "NULL";
         $field_nuke = "KILL";
         $field_oblivion = "NO";
       }
     }
   }
   close (FILE);
   return "";
  }
}
1;
